{{

  **************************************************************************************
  **            Britt Tool, Inc -- Door Lock System v1.0                              **
  **            By: Rick Finkbiner                                                    **
  **            September 15, 2008                                                    **
  **************************************************************************************
  
}}

CON

  _clkmode      =               xtal1 + pll16x
  _xinfreq      =               5_000_000
  
  Xport_rx      =               0
  Xport_tx      =               1
  RFID_Enable   =               8
  RFID_Serial   =               9
  IR_LED        =               11
  IR_Detect     =               12
  DoorStatus    =               13   
  Door          =               20
  
VAR

  LONG IR_Cog, RFID_Cog, Xport_Cog
  LONG IR_Stack[20], RFID_Stack[30], Xport_Stack[30]
  BYTE ObjectFound, GotRFID
  BYTE RFID_Code[11]

OBJ

  Serial        :               "Simple_Serial"
  Xport         :               "FullDuplexSerial"
  
PUB Main

  Ini
  Xport_Cog := cognew(XportLink, @Xport_Stack[0]) + 1                           ' Start Network cog
  repeat
    IR_Cog := cognew(MakeIR, @IR_Stack[0]) + 1                                  ' start IR beam cog
    ScanObject                                                                  ' wait for object
    cogstop(IR_Cog~ - 1)                                                        ' stop IR beam
    RFID_Cog := cognew(RFIDScan, @RFID_Stack[0]) + 1                            ' start RFID cog
    repeat 24                                                                   ' wait up to 6 seconds for an RFID
      waitcnt(clkfreq / 4 + cnt)
      if GotRFID == 1
        quit
    cogstop(RFID_Cog~ - 1)
    waitcnt(clkfreq + cnt)
    
PRI Ini
         
  dira[IR_Detect] := 0
  ObjectFound := GotRFID := 0
  
PRI MakeIR     

  dira[IR_LED] := 1                                                             ' IR beam for the object detector
  repeat                                                                        ' 36.7kHz signal
    !outa[IR_LED]
    waitcnt(clkfreq / 75_000 + cnt) 
    
PRI ScanObject

  repeat                                                                        ' Object is detected via IR reflection
    if ina[IR_Detect] == 0
      waitcnt(clkfreq / 10 + cnt)                                               ' if object is there for more than 1/10th of a second
      if ina[IR_Detect] == 0                                                    ' set the ObjectFound flag
        ObjectFound := 1
        quit
      else
        ObjectFound := 0
          
PRI RFIDScan | Index, Baud

  dira[RFID_Enable] := 1                                                        ' Sets the RFID ports for communication
  dira[RFID_Serial] := 0                                         
  outa[RFID_Enable] := 0
  Baud := 2400                                                                  ' Sets the serial baud rate to 2400
  Serial.init(RFID_Serial, -1, Baud)                                            ' Starts Serial communication  
  WaitForSignal
  repeat Index from 0 to 9                                                      ' Load the RFID data into the buffer
    RFID_Code[Index] := Serial.rx
  outa[RFID_Enable] := 1                                                        ' Stops the RFID reader from getting data
  Serial.finalize
  RFID_Code[10] := 0
  if RFID_Code[9] == "0" and RFID_Code[8] == "0" and RFID_Code[7] == "0" and RFID_Code[6] == "0"
    bytefill(@RFID_Code, 0, 11)                                                 ' if the RFID code is mostly 0's wipe the data out
  else 
    GotRFID := 1
    
PRI WaitForSignal 
                                                                                ' Loops until you have data
  repeat
    if Serial.rx == $0A                                                         ' Exit the loop when you get the start code ($0A)
      quit
      
PRI OpenDoor

  dira[Door] := 1                                                               ' open the door for 8 seconds
  outa[Door] := 1
  waitcnt(clkfreq * 3 + cnt)
  outa[Door] := 0

PRI XportLink | Index , MyVal

  dira[DoorStatus] := 0
  Xport.start(Xport_rx,Xport_tx, $0000, 9600)                                   ' start serial comm to the Xport
  repeat
    if GotRFID == 1                                                             ' transmit any RFID data to the server
      repeat Index from 0 to 9
        Xport.tx(RFID_Code[Index]) 
      Xport.tx(13) 
      GotRFID := 0
      bytefill(@RFID_Code, 0, 11)
    MyVal := Xport.rxtime(200)
    if MyVal == "Y"                                                             ' if any data is received process data
      OpenDoor
    elseif MyVal == "S"
      if ina[DoorStatus] == 0
        Xport.tx("C")
      else
        Xport.tx("O")
      Xport.tx(13)
        